#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

constexpr byte Char_Remain = Char_Fighter;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

static ptr<byte> PrintRemain(ptr<byte> pVram)
{
    byte c;
    c = Char_Remain;
    repeat (2) {
        pVram = PutU(pVram, c);
        ++c;
    }
    pVram -= 2 * VramStep;
    repeat (2) {
        pVram = PutL(pVram, c);
        ++c;
    }
    return pVram;
}

static ptr<byte> EraseRemain(ptr<byte> pVram)
{
    repeat (2) {
        pVram = PrintC(pVram, ' ');
    }
    return pVram;
}


void PrintStatus() 
{
    PrintS(Vram + 0 * VramStep, "SC");   
    PrintS(Vram + 10 * VramStep, "ST");
    {
        ptr<byte> pVram;
        pVram = Vram + 20 * VramStep;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                pVram = PrintRemain(pVram);
                pVram = PrintC(pVram, ' ');
                pVram = PrintC(pVram, i + '0');
            }
            else {
                do {
                    pVram = PrintRemain(pVram);
                    --i;
                } while (i != 0);
                EraseRemain(pVram);
            }
        }
        else {
            EraseRemain(pVram);           
        }
    }
    PrintStage();
    PrintScore();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + 2 * VramStep, Score);
    PrintC(pVram, '0');

    // pVram = PrintNumber5(Vram + 9 * VramStep, HiScore);
    // PrintC(pVram, '0');
}

void PrintStage()
{
    PrintByteNumber2(Vram + 12 * VramStep, CurrentStage + 1);
}

void PrintGameOver()
{
    PrintS(Vram + VramRowSize * 4 + 8 * VramStep, "GAME OVER");
    PresendVram();
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    PrintStatus(); 
    {
        static const byte[] TitleBytes = {
            //	A
            0x00, 0x0e, 0x0d, 0x02, 
            0x0c, 0x03, 0x00, 0x0f, 
            0x0c, 0x07, 0x05, 0x0f, 
            0x04, 0x01, 0x00, 0x05, 
            //	E
            0x0c, 0x07, 0x05, 0x01, 
            0x0c, 0x0b, 0x0a, 0x02, 
            0x0c, 0x03, 0x00, 0x00, 
            0x04, 0x05, 0x05, 0x01, 
            //	R
            0x0c, 0x07, 0x05, 0x0b, 
            0x0c, 0x03, 0x08, 0x0f, 
            0x0c, 0x07, 0x0f, 0x02, 
            0x04, 0x01, 0x04, 0x05, 
            //	I
            0x04, 0x0d, 0x07, 0x01, 
            0x00, 0x0c, 0x03, 0x00, 
            0x00, 0x0c, 0x03, 0x00, 
            0x04, 0x05, 0x05, 0x01, 
            //	A
            0x00, 0x0e, 0x0d, 0x02, 
            0x0c, 0x03, 0x00, 0x0f, 
            0x0c, 0x07, 0x05, 0x0f, 
            0x04, 0x01, 0x00, 0x05, 
            //	L
            0x0c, 0x03, 0x00, 0x00, 
            0x0c, 0x03, 0x00, 0x00, 
            0x0c, 0x03, 0x00, 0x00, 
            0x04, 0x05, 0x05, 0x01, 
        };
        constexpr byte LogoLength = 6;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 1 + (VramWidth - LogoLength * 4) * VramStep / 2;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (2) {
                repeat (4) {
                    pVram = PutU(pVram, *p);
                    ++p;
                }
                pVram -= 4 * VramStep;
                repeat (4) {
                    pVram = PutL(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 2;
        }
        PrintS(Vram + VramRowSize * 3 + (VramWidth / 2 + LogoLength * 2 - 4) * VramStep, "MINI");
    }

    PrintS(Vram + VramRowSize * 5 + 5 * VramStep, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 6 + 5 * VramStep, "OR RETURN KEY");
    PrintS(Vram + VramRowSize * 7 + 12 * VramStep, "INUFUTO 2023");

    PresendVram();
}